/*
 *
 *  85xx_dma.h, version 1.0 
 *  Public API for general purpose DMA controller
 *
 *  This software may be used and distributed according to the terms of
 *  the GNU Public License, Version 2, incorporated herein by reference.
 *
 *  Contact:  <kumar.gala@motorola.com>
 *
 */

#ifndef __MPC85XX_DMA_H__
#define __MPC85XX_DMA_H__

#include <linux/list.h>
#include <asm/ocp.h>

#define DMA_MODE_SHARED		0	/* multi users for the channel */
#define DMA_MODE_EXCLUSIVE	1	/* a single user of the channel */
#define DMA_MODE_STRIDING	2	/* enable striding features on 
					   channel, orthogonal to 
					   SHARED & EXCLUSIVE */

struct dma_pub {
	struct list_head list;
	void *src;		/* virtual source address for transfer */
	void *dst;		/* virtual destination address for transfer */
	size_t size;		/* size in bytes for transfer */
	int direction;		/* not used -- future */
	void (*callback) (void *);	/* if non-null, will be called after transfer has
					   occured */
	int status;		/* status code returned, only valid in callback */
	char usr[32];		/* user defined field */
};

/* Each channel is independent.  A handle based system is used for
 * all other API calls. */
extern int mpc85xx_dma_open(int num, int mode, void **dma_ch);
extern void mpc85xx_dma_close(void *dma_ch);

/* descriptors are owned by the channels.  This is to abstract the
 * true hardware descriptors and list mechanisms from the user */
extern int mpc85xx_dma_alloc_desc(struct dma_pub **desc, void *dma_ch);

/* If the user over allocated a desc, return it to the channel free pool */
extern int mpc85xx_dma_free_desc(struct dma_pub *desc, void *dma_ch);

/* queue a single descriptor to be transfered, will be returned to free pool */
extern int mpc85xx_dma_queue_desc(struct dma_pub *pub, void *dma_ch);

/* get a list of descriptors from the channel, uses same dma_pub descriptors
 * but allow list managemnt of them to the user */
extern int mpc85xx_dma_alloc_list(unsigned int count, struct list_head *head,
				  void *dma_ch);
extern int mpc85xx_dma_free_list(struct list_head *head, void *dma_ch);
extern int mpc85xx_dma_queue_list(struct list_head *head, void *dma_ch);

extern int mpc85xx_dma_set_striding_info(void *dma_ch, u16 src_size,
					 u16 src_dist, u16 dst_size,
					 u16 dst_dist);

#endif				/* __MPC85XX_DMA_H__ */
