/*
 * A collection of structures, addresses, and values associated with
 * the DTI ATS-1460 boards.
 * Copied from the RPX-Classic and SBS8260 stuff.
 *
 * Copyright (c) 2001 Dan Malek (dan@mvista.com)
 */
#ifdef __KERNEL__
#ifndef __MACH_ATS1460_DEFS
#define __MACH_ATS1460_DEFS

#include <linux/config.h>
#include <asm/ppcboot.h>

#define IMAP_ADDR               ((uint)0xf0000000)

#define PHY_INTERRUPT           (SIU_INT_IRQ6)

#ifdef CONFIG_PCI
/* PCI interrupt controller */
#define PIRQA                   (0x14)

/*
 * PCI memory map definitions for ATS1460-PCI.
 *
 * processor view
 *  local address           PCI address             target
 *  0x80000000-0x9FFFFFFF   0x80000000-0x9FFFFFFF   PCI mem with prefetch
 *  0xA0000000-0xBFFFFFFF   0xA0000000-0xBFFFFFFF   PCI mem w/o prefetch
 *  0xF4000000-0xF7FFFFFF   0x00000000-0x03FFFFFF   PCI IO
 *
 * PCI master view
 *  local address           PCI address             target
 *  0x00000000-0x1FFFFFFF   0x00000000-0x1FFFFFFF   MPC8200 local memory
 */

/* window for a PCI master to access MPC8200 memory */
#define PCI_SLV_MEM_LOCAL       0x00000000    /* Local base */
#define PCI_SLV_MEM_BUS         0x00000000    /* PCI base */

/* window for the processor to access PCI memory with prefetching */
#define PCI_MSTR_MEM_LOCAL      0x80000000    /* Local base */
#define PCI_MSTR_MEM_BUS        0x80000000    /* PCI base   */
#define PCI_MSTR_MEM_SIZE       0x20000000    /* 512MB */

/* window for the processor to access PCI memory without prefetching */
#define PCI_MSTR_MEMIO_LOCAL    0xA0000000    /* Local base */
#define PCI_MSTR_MEMIO_BUS      0xA0000000    /* PCI base   */
#define PCI_MSTR_MEMIO_SIZE     0x20000000    /* 512MB */

/* window for the processor to access PCI I/O */
#define PCI_MSTR_IO_LOCAL       0xF4000000    /* Local base */
#define PCI_MSTR_IO_BUS         0x00000000    /* PCI base   */
#define PCI_MSTR_IO_SIZE        0x04000000    /* 64MB */

#define _IO_BASE                PCI_MSTR_IO_LOCAL
#define _ISA_MEM_BASE           PCI_MSTR_MEMIO_LOCAL
#define PCI_DRAM_OFFSET         PCI_SLV_MEM_BUS
#endif /* CONFIG_PCI */

#endif /* __MACH_ATS1460_DEFS */
#endif /* __KERNEL__ */
