/*
 * arch/ppc/platforms/ng_gsm7248.c
 *
 * Board setup routines for Netgear GSM7248
 *
 * Author: Aharon Gadot aharong@radlan.com
 *
 * 2004 (c) Radlan A Marvell company ltd.
 *
 * This file is licensed under
 * the terms of the GNU General Public License version 2.  This program
 * is licensed "as is" without any warranty of any kind, whether express
 * or implied.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/irq.h>
#include <linux/ide.h>
#include <linux/irq.h>
#include <linux/seq_file.h>
#include <linux/serial.h>
#include <linux/pci_ids.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/time.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/prom.h>
#include <asm/smp.h>
#include <asm/open_pic.h>
#include <asm/todc.h>
#include <asm/bootinfo.h>
#include <asm/mpc10x.h>
#include <asm/pci-bridge.h>
#include <asm/ppcboot.h>

#include "ng_gsm7248_serial.h"
#include "ng_gsm7248.h"

extern void gen550_progress(char *, unsigned short);
extern void gen550_init(int, struct serial_struct *);

unsigned char __res[sizeof (bd_t)];

static void ng_gsm7248_halt(void);

static u_char ng_gsm7248_openpic_initsenses[] __initdata = {
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 2 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 3 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 4 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 5 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),      /* IRQ 6 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_POSITIVE),      /* IRQ 7 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 8 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 9 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 10 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 11 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 12 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 13 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 14 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* IRQ 15 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* I2C */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* DMA 0 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* DMA 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* MSG Unit */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE),      /* DUART 1 */
	(IRQ_SENSE_LEVEL | IRQ_POLARITY_NEGATIVE)       /* DUART 2 */
};

/*
 * PCI Interrupt routing.
 */
static inline int
ng_gsm7248_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin)
{
	static char pci_irq_table[][4] =
	/*
	 *      PCI IDSEL/INTPIN->INTLINE
	 *         A   B   C   D
	 */
	{
			{ 1, 1, 1, 1 },		/* IDSEL 16/0x10 - DX260 */
			{ 1, 1, 1, 1 },		/* IDSEL 17/0x11 - DX260 */
	};

	const long min_idsel = 16, max_idsel = 17, irqs_per_slot = 1;
	return PCI_IRQ_TABLE_LOOKUP;

	/*
    if ((dev->vendor == PCI_VENDOR_ID_NS) &&
        (dev->device == PCI_DEVICE_ID_NS_83815))
    return NG_GSM7248_ETHERNET_IRQ;
    
    return NG_GSM7248_PCI_IRQ;
	*/
}
       
static void __init
ng_gsm7248_find_bridges(void)
{
	struct pci_controller *hose;

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno  = 0xff;

	if (mpc10x_bridge_init(hose,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MEM_MAP_B, 0xfc000000) == 0) {

		if (ppc_md.progress)
			ppc_md.progress("After Bridge init", 0x100);
		/* Scan PCI bus */
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

		printk("PCI Auto Scan done\n");
		if (ppc_md.progress)
			ppc_md.progress("After bus scan", 0x100);

		ppc_md.pcibios_fixup     = NULL;
		ppc_md.pcibios_fixup_bus = NULL;
		ppc_md.pci_swizzle       = common_swizzle;
		ppc_md.pci_map_irq       = ng_gsm7248_map_irq;
		if (ppc_md.progress)
			ppc_md.progress("Bridge init worked", 0x100);
	} else {
		if (ppc_md.progress)
			ppc_md.progress("Bridge init failed", 0x100);
		printk("Host bridge init failed\n");
	}

	return;
}

#ifdef CONFIG_SERIAL
unsigned int ng_gsm7248_serial = 0xfc004500;

static void __init
ng_gsm7248_early_serial_map(void)
{
	struct serial_struct serial_req;

	/* Setup serial port access */
	memset(&serial_req, 0, sizeof (serial_req));

	serial_req.baud_base        = BASE_BAUD;
	serial_req.line             = 0;
	serial_req.port             = 0;
	serial_req.irq              = NG_GSM7248_SERIAL_0_IRQ;
	serial_req.flags            = STD_COM_FLAGS;
	serial_req.io_type          = SERIAL_IO_MEM;
	serial_req.iomem_reg_shift  = 0;

    serial_req.iomem_base = ioremap(NG_GSM7248_SERIAL_0, 0x20);  

    ng_gsm7248_serial = serial_req.iomem_base;
#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	gen550_init(0, &serial_req);
#endif

#if 0
	if (early_serial_setup(&serial_req) != 0)
		printk("Early serial init of port 0 failed\n");
#endif

}
#endif

static void __init
ng_gsm7248_setup_arch(void)
{
	loops_per_jiffy = 100000000 / HZ;

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
	else
#endif
#ifdef	CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x0301);	/* /dev/hda1 IDE disk */
#endif

		if (ppc_md.progress)
			ppc_md.progress("bridge init", 0x100);
	/* Lookup PCI host bridges */
	ng_gsm7248_find_bridges();
	if (ppc_md.progress)
		ppc_md.progress("bridge init done", 0x100);

#ifdef CONFIG_SERIAL
	ng_gsm7248_early_serial_map();
#endif

	if (ppc_md.progress)
		ppc_md.progress("serial map done", 0x100);

    /* TODC_INIT(TODC_TYPE_DS1501, 0, 0, ioremap(NG_GSM7248_TODC_BASE, NG_GSM7248_TODC_SIZE), 8);
	 * */

#ifdef CONFIG_DUMMY_CONSOLE
	conswitchp = &dummy_con;
#endif

	if (ppc_md.progress)
		ppc_md.progress("TODC done", 0x100);
	printk(KERN_INFO "NetGear GSM7248\n");
	printk(KERN_INFO "GSM7248 port (C) 2005 LVL7 Systems, Inc. "
					 "(asusarla@lvl7.com)\n");
}
 
static void __init
ng_gsm7248_init2(void)
{
	/* Do board specific initialization.  */

	request_region(0x00, 0x20, "dma1");
	request_region(0x20, 0x20, "pic1");
	request_region(0x40, 0x20, "timer");
	request_region(0x80, 0x10, "dma page reg");
	request_region(0xa0, 0x20, "pic2");
	request_region(0xc0, 0x20, "dma2");

	return;
}

/*
 * Interrupt setup and service. 
 */
static void __init
ng_gsm7248_init_IRQ(void)
{
    int i;

	OpenPIC_InitSenses    = ng_gsm7248_openpic_initsenses;
	OpenPIC_NumInitSenses = sizeof (ng_gsm7248_openpic_initsenses);

	/*
	 * We need to tell openpic_set_sources where things actually are.
	 * mpc10x_common will setup OpenPIC_Addr at ioremap(EUMB phys base +
	 * EPIC offset (0x40000));  The EPIC IRQ Register Address Map -
	 * Interrupt Source Configuration Registers gives these numbers
	 * as offsets starting at 0x50200, we need to adjust occordinly.
	 */

     /* Map 16 serial IRQs */
     openpic_set_sources(NG_GSM7248_FIRST_IRQ, 
                         NG_GSM7248_NUM_OF_IRQS, 
			 OpenPIC_Addr + 0x10200);
	 /* Skip reserved space and map i2c and DMA Ch[01] */
	 openpic_set_sources(16, 3, OpenPIC_Addr + 0x11020);
	 /* Skip reserved space and map Message Unit Interrupt (I2O) */
	 openpic_set_sources(19, 1, OpenPIC_Addr + 0x110C0);
	 /* Skip reserved space and map DUART */
	 openpic_set_sources(20, 2, OpenPIC_Addr + 0x11120);

     /* init openpic wtih first IRQ number - 0 */
     openpic_init(NG_GSM7248_FIRST_IRQ);
}

static void __init
ng_gsm7248_calibrate_decr(void)
{
	bd_t    *binfo = (bd_t *)__res;
    ulong freq;

    /* freq = (ulong)33000000; */
	freq = (binfo->bi_busfreq / 4);

	printk("time_init: decrementer frequency = %lu.%.6lu MHz\n",
	           freq/1000000, freq%1000000);

	tb_ticks_per_jiffy = freq / HZ;
	tb_to_us = mulhwu_scale_factor(freq, 1000000);
#ifdef CONFIG_ILATENCY
	{
	   extern unsigned ticks_per_usec;
	   ticks_per_usec = freq / 1000000;
	}
#endif

	return;
}

static u32
ng_gsm7248_irq_cannonicalize(u32 irq)
{
    return irq;
}

static unsigned long __init
ng_gsm7248_find_end_of_memory(void)
{
	bd_t *binfo;
	extern unsigned char __res[];

	binfo = (bd_t *)__res;

	return binfo->bi_memsize;
	/* return 64 * 1024 * 1024; */
}

static void __init
ng_gsm7248_map_io(void)
{
	io_block_mapping(0xfc000000, 0xfc000000, 0x00100000, _PAGE_IO);
	io_block_mapping(0xfec00000, 0xfec00000, 0x00400000, _PAGE_IO);
}

inline void out_le32 (volatile unsigned *addr, int val)
{
	    __asm__ __volatile__ ("stwbrx %1,0,%2; eieio":"=m" (*addr):"r" (val),
				                  "r" (addr));
}

#define writedata(val, addr) out_le32(addr, val)

static void
ng_gsm7248_restart(char *cmd)
{
	__cli();

	writedata (0x80 | 0x20 | 0x10, i2cccr);
	writedata (0, i2ccdr);

	for (;;);		/* Spin until reset happens */
}

static void
ng_gsm7248_power_off(void)
{
	__cli();
	for (;;) ;		/* No way to shut power off with software */
	/* NOTREACHED */
}

static void
ng_gsm7248_halt(void)
{
	ng_gsm7248_power_off();
	/* NOTREACHED */
}

static int
ng_gsm7248_show_cpuinfo(struct seq_file *m)
{
	bd_t *bp;

	bp = (bd_t *)__res;
	
	seq_printf(m, "vendor\t\t: %s\n" ,BOARD_VENDOR);
	seq_printf(m, "machine\t\t: %s\n",BOARD_MACHINE);
	seq_printf(m, "cpu speed\t: %ldMhz\n", bp->bi_intfreq/1000000);
	seq_printf(m, "bus speed\t: %ldMhz\n", bp->bi_busfreq/1000000);

	return 0;
}


/*
 * Set BAT 3 to map 0xf8000000 to end of physical memory space 1-to-1.
 */
static __inline__ void
ng_gsm7248_set_bat(void)
{
	mb();
	mtspr(DBAT1U, 0xf8000ffe);
	mtspr(DBAT1L, 0xf800002a);
	mb();
}

TODC_ALLOC();

void __init 
platform_init(unsigned long r3, unsigned long r4, unsigned long r5,
	      unsigned long r6, unsigned long r7)
{
	parse_bootinfo(find_bootinfo());

	/* ASSUMPTION:  If both r3 (bd_t pointer) and r6 (cmdline pointer)
	 * are non-zero, then we should use the board info from the bd_t
	 * structure and the cmdline pointed to by r6 instead of the
	 * information from birecs, if any.  Otherwise, use the information
	 * from birecs as discovered by the preceeding call to
	 * parse_bootinfo().  This rule should work with both PPCBoot, which
	 * uses a bd_t board info structure, and the kernel boot wrapper,
	 * which uses birecs.
	 */
	
	 if (r3 && r6) {
	 	/* copy board info structure */
	 	memcpy((void *) __res, (void *) (r3 + KERNELBASE),
	 	       sizeof (bd_t));
	 	/* copy command line */ 
	 	*(char *) (r7 + KERNELBASE) = 0;
	 	strcpy(cmd_line, (char *) (r6 + KERNELBASE));
	  }
	
#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if (r4) {
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif				/* CONFIG_BLK_DEV_INITRD */

	/* Map in board regs, etc. */
	ng_gsm7248_set_bat();

	isa_io_base = MPC10X_MAPB_ISA_IO_BASE;
	isa_mem_base = MPC10X_MAPB_ISA_MEM_BASE;
	pci_dram_offset = MPC10X_MAPB_DRAM_OFFSET;
	ISA_DMA_THRESHOLD = 0x00ffffff;
	DMA_MODE_READ = 0x44;
	DMA_MODE_WRITE = 0x48;

	ppc_md.setup_arch = ng_gsm7248_setup_arch;
	ppc_md.show_cpuinfo = ng_gsm7248_show_cpuinfo;
	ppc_md.irq_cannonicalize = ng_gsm7248_irq_cannonicalize;
	ppc_md.init_IRQ = ng_gsm7248_init_IRQ;
	ppc_md.get_irq = openpic_get_irq;
	ppc_md.init = NULL;

	ppc_md.restart = ng_gsm7248_restart;
	ppc_md.power_off = ng_gsm7248_power_off;
	ppc_md.halt = ng_gsm7248_halt;

	ppc_md.find_end_of_memory = ng_gsm7248_find_end_of_memory;
	ppc_md.setup_io_mappings = ng_gsm7248_map_io;

	
	ppc_md.time_init = NULL;
	ppc_md.set_rtc_time = NULL;
	ppc_md.get_rtc_time = NULL;
	/* ppc_md.calibrate_decr = todc_calibrate_decr; */
	ppc_md.calibrate_decr = ng_gsm7248_calibrate_decr;

	ppc_md.nvram_read_val = NULL;
	ppc_md.nvram_write_val = NULL;

#ifdef CONFIG_SERIAL
#ifdef CONFIG_SERIAL_TEXT_DEBUG
	ppc_md.progress = gen550_progress;
#endif
	ppc_md.early_serial_map = ng_gsm7248_early_serial_map;
#endif

	return;
}
