/*
 * $Id: fsm73xx.c,v 1.2 2005/07/14 20:09:28 rnarayanan Exp $
 *
 * Normal mappings of chips in physical memory
 *
 * Modified for Netgear FSM73xx partitions... -- JWL
 */

/* partition support */
#define HAVE_PARTITIONS

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/config.h>
#ifdef HAVE_PARTITIONS
#include <linux/mtd/partitions.h>
#endif

#define WINDOW_ADDR	0xff000000
#define WINDOW_SIZE	0x01000000
#define BUSWIDTH	1

#ifdef CONFIG_RIVERSTONE_ES2010
#define APP_START       0x00000000
#define APP_LEN         0x00dc0000 /* Was 0xde0000 - Ram (06/29/05) */

#define SYSTEM_START    0x00dc0000 /* Was 0xde0000 - Ram (06/29/05) */
#define SYSTEM_LEN      0x00140000 /* Was 0x120000 - Ram (06/29/05) */

#else

#define APP_START	0x00000000
#define APP_LEN		0x00de0000

#define SYSTEM_START	0x00de0000
#define SYSTEM_LEN	0x00120000
#endif /* CONFIG_RIVERSTONE_ES2010 */

#define U_BOOT_START	0x00f00000
#define U_BOOT_LEN	0x00020000

#define ENV_START	0x00f20000
#define ENV_LEN		0x00020000

#define VPD_START	0x00f40000
#define VPD_LEN		0x00020000

#define SYSTEM_2_START	0x00f60000
#define SYSTEM_2_LEN	0x000a0000

#ifdef HAVE_PARTITIONS
static struct mtd_partition fsm73xx_partitions[] =
{
	/* app filesystem */
	{
		      .name = "Application Filesystem",
		    .offset = APP_START,
		      .size = APP_LEN,
		.mask_flags = 0
	},
	/* system (kernel + ramdisk) */
	{
		      .name = "System (Kernel + Ramdisk)",
		    .offset = SYSTEM_START,
		      .size = SYSTEM_LEN,
		.mask_flags = 0
	},
	/* u-boot */
	{
		      .name = "U-Boot",
		    .offset = U_BOOT_START,
		      .size = U_BOOT_LEN,
		.mask_flags = 0
	},
	/* u-boot environment */
	{
		      .name = "U-Boot Environment",
		    .offset = ENV_START,
		      .size = ENV_LEN,
		.mask_flags = 0
	},
	/* VPD */
	{
		      .name = "FASTPATH Vital Product Data",
		    .offset = VPD_START,
		      .size = VPD_LEN,
		.mask_flags = 0
	},
	/* system (part 2) */
	{
		      .name = "System (part 2)",
		    .offset = SYSTEM_2_START,
		      .size = SYSTEM_2_LEN,
		.mask_flags = 0
	}
};

static struct mtd_partition fsm73xx_partitions_2[] =
{
	/* app filesystem */
	{
		      .name = "Application Filesystem",
		    .offset = APP_START,
		      .size = APP_LEN - (WINDOW_SIZE / 2),
		.mask_flags = 0
	},
	/* system (kernel + ramdisk) */
	{
		      .name = "System (Kernel + Ramdisk)",
		    .offset = SYSTEM_START - (WINDOW_SIZE / 2),
		      .size = SYSTEM_LEN,
		.mask_flags = 0
	},
	/* u-boot */
	{
		      .name = "U-Boot",
		    .offset = U_BOOT_START - (WINDOW_SIZE / 2),
		      .size = U_BOOT_LEN,
		.mask_flags = 0
	},
	/* u-boot environment */
	{
		      .name = "U-Boot Environment",
		    .offset = ENV_START - (WINDOW_SIZE / 2),
		      .size = ENV_LEN,
		.mask_flags = 0
	},
	/* VPD */
	{
		      .name = "FASTPATH Vital Product Data",
		    .offset = VPD_START - (WINDOW_SIZE / 2),
		      .size = VPD_LEN,
		.mask_flags = 0
	},
	/* system (part 2) */
	{
		      .name = "System (part 2)",
		    .offset = SYSTEM_2_START - (WINDOW_SIZE / 2),
		      .size = SYSTEM_2_LEN,
		.mask_flags = 0
	}
};

#define NUM_PARTITIONS	(sizeof(fsm73xx_partitions) \
                       / sizeof(struct mtd_partition))
#endif

static struct mtd_info *mymtd;

struct map_info fsm73xx_map = {
	.name = "Physically mapped flash",
	.phys = WINDOW_ADDR,
	.size = WINDOW_SIZE,
	.buswidth = BUSWIDTH
};

int __init init_fsm73xx(void)
{
       	printk(KERN_NOTICE "fsm73xx flash device: %x at %x\n", WINDOW_SIZE, WINDOW_ADDR);
	fsm73xx_map.virt = (unsigned long)ioremap(WINDOW_ADDR, WINDOW_SIZE);

	if (!fsm73xx_map.virt) {
		printk("Failed to ioremap\n");
		return -EIO;
	}

	simple_map_init(&fsm73xx_map);

	mymtd = do_map_probe("cfi_probe", &fsm73xx_map);
	if (mymtd) {
		mymtd->owner = THIS_MODULE;

#if defined(HAVE_PARTITIONS)
		add_mtd_partitions(mymtd, fsm73xx_partitions,
		                   NUM_PARTITIONS);
#else
		add_mtd_device(mymtd);
#endif
		return 0;
	} else { /* Some boxes have 1/2 flash... */
		iounmap((void *)fsm73xx_map.virt);
		fsm73xx_map.virt
		   = (unsigned long)ioremap(WINDOW_ADDR + (WINDOW_SIZE / 2),
		                            (WINDOW_SIZE / 2));
		fsm73xx_map.phys += (WINDOW_SIZE / 2);
		fsm73xx_map.size = (WINDOW_SIZE / 2);

		mymtd = do_map_probe("cfi_probe", &fsm73xx_map);
		if (mymtd) {
			mymtd->owner = THIS_MODULE;

#if defined(HAVE_PARTITIONS)
			add_mtd_partitions(mymtd, fsm73xx_partitions_2,
			                   NUM_PARTITIONS);
#else
			add_mtd_device(mymtd);
#endif
			return 0;
		}
	}

	iounmap((void *)fsm73xx_map.virt);
	return -ENXIO;
}

static void __exit cleanup_fsm73xx(void)
{
	if (mymtd) {
#if defined(HAVE_PARTITIONS)
		del_mtd_device(mymtd);
#else
		del_mtd_partitions(mymtd);
#endif
		map_destroy(mymtd);
	}
	if (fsm73xx_map.virt) {
		iounmap((void *)fsm73xx_map.virt);
		fsm73xx_map.virt = 0;
	}
}

module_init(init_fsm73xx);
module_exit(cleanup_fsm73xx);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("John W. Linville <linville@lvl7.com>");
MODULE_DESCRIPTION("MTD map driver for Netgear FSM73xx");
